@@ -21,7 +21,6 @@ class VideoWidget(MediaWidget):
     Media widget for viewing videos.
     """
 
-
     def __init__(self):
     
         self.__layout = None
@@ -30,6 +29,7 @@ class VideoWidget(MediaWidget):
         mediaplayer.add_observer(self.__on_observe_player)
 
         self.__aspect_ratio = 1.0
+        self.__overlay_coords = (0, 0, 0, 0)
 
         self.__current_file = None
         self.__uri = ""
@@ -39,8 +39,8 @@ class VideoWidget(MediaWidget):
       
         # video screen
         self.__screen = gtk.DrawingArea()
-        self.__screen.hide()
         self.__screen.set_double_buffered(False)
+        self.__screen.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))
         self.__screen.connect("expose-event", self.__on_expose)
         self.__screen.set_events(gtk.gdk.BUTTON_PRESS_MASK |
                                  gtk.gdk.KEY_PRESS_MASK)
@@ -70,14 +70,13 @@ class VideoWidget(MediaWidget):
         
         self._set_controls(Image(theme.mb_toolbar_space_1),
                            self.__btn_play,
-                           Image(theme.mb_toolbar_space_2),
+                           Image(theme.mb_progress_left),
                            self.__progress,
-                           Image(theme.mb_toolbar_space_2),
+                           Image(theme.mb_progress_right),
                            btn_bookmark,
                            Image(theme.mb_toolbar_space_1))        
 
 
-
     def handle_message(self, msg, *args):
     
         if (msg == msgs.INPUT_EV_NAV_UP):
@@ -98,6 +97,11 @@ class VideoWidget(MediaWidget):
         elif (msg == msgs.INPUT_EV_NAV_MENU):
             handled = self.__player.send_key(self.__player.KEY_MENU1)
 
+        elif (msg == msgs.MEDIA_EV_DOWNLOAD_PROGRESS):
+            f, amount, total = args
+            if (f == self.__current_file and total > 0):
+                self.__progress.set_amount(float(amount) / total)
+                
 
     def _visibility_changed(self):
 
@@ -117,8 +121,8 @@ class VideoWidget(MediaWidget):
     def __show_video_screen(self):
     
         if (not self.get_screen().is_offscreen() and
-            self.may_render() and
-            self.__player):
+              self.may_render() and
+              self.__player):
             self.__screen.show()
             
     
@@ -137,16 +141,41 @@ class VideoWidget(MediaWidget):
         self.__ebox.set_geometry(0, 0, w, h)
 
         fx, fy, fw, fh = x, y, w, h
-        vx, vy, vw, vh = x + 2, y + 2, w - 4, h - 4
+        vx, vy, vw, vh = x + 1, y + 1, w - 2, h - 2
+        #vx, vy, vw, vh = x, y, w, h
 
         if (w < 800):
-            screen.draw_rect(fx, fy, fw, fh, "#000000")
-            screen.fill_area(vx, vy, vw, vh, "#000000")
+            self.__render_overlay_frame(screen, fx, fy, fw, fh)
+            screen.draw_rect(vx, vy, vw, vh, theme.color_mb_background)
+            #screen.fill_area(vx, vy, vw, vh, "#000000")
         else:
-            screen.fill_area(x, y, w, h, "#000000")
+            self.__render_overlay_frame(screen, fx, fy, fw, fh)
+        #    screen.fill_area(x, y, w, h, "#000000")
 
         gobject.idle_add(self.__show_video_screen)
 
+
+    def __render_overlay_frame(self, screen, x, y, w, h):
+    
+        vx, vy, vw, vh = self.__overlay_coords
+        screen.fill_area(x, y, w, h, "#000000")
+        return
+        
+        w1 = vx - x
+        h1 = vy - y
+        
+        if (w1 > 0):
+            # left
+            screen.fill_area(x, y, w1, h, "#000000")
+            # right
+            screen.fill_area(x + w - w1, y, w1, h, "#000000")
+            
+        if (h1 > 0):
+            # top
+            screen.fill_area(x, y, w, h1, "#000000")
+            # bottom
+            screen.fill_area(x, y + h - h1, w, h1, "#000000")
+        
         
     def __on_expose(self, src, ev):
 
@@ -165,8 +194,8 @@ class VideoWidget(MediaWidget):
         if (self.__player):
             handled = self.__player.send_key(self.__player.KEY_SELECT)
             
-        if (not handled):
-            self.send_event(self.EVENT_FULLSCREEN_TOGGLED)
+            if (not handled):
+                self.send_event(self.EVENT_FULLSCREEN_TOGGLED)
 
         
 
@@ -186,8 +215,8 @@ class VideoWidget(MediaWidget):
                 else:
                     info = "%d:%02d" % (pos_m, pos_s)            
 
-                self.send_event(self.EVENT_MEDIA_POSITION, info)
                 self.__progress.set_position(pos, total)
+                self.send_event(self.EVENT_MEDIA_POSITION, info)
 
             # bad mplayer shows its screen even when not asked for...
             #if (not self.may_render()):
@@ -196,6 +225,7 @@ class VideoWidget(MediaWidget):
         elif (cmd == src.OBS_STARTED):
             #print "Started Player"
             self.__progress.set_message("")
+            self.__progress.set_amount(0)
             #self.__btn_play.set_images(theme.btn_play_1,
             #                           theme.btn_play_2)
             
@@ -203,6 +233,7 @@ class VideoWidget(MediaWidget):
             #print "Killed Player"
             self.__uri = ""
             self.__progress.set_message("")
+            self.__progress.set_amount(0)
             self.__hide_video_screen()
             self.__btn_play.set_images(theme.mb_btn_play_1,
                                        theme.mb_btn_play_2)
@@ -211,6 +242,7 @@ class VideoWidget(MediaWidget):
             ctx = args[0]
             if (ctx == self.__context_id):
                 self.__progress.set_message("... connecting ...")
+                self.__progress.set_amount(0)
 
         elif (cmd == src.OBS_BUFFERING):
             ctx, value = args
@@ -258,8 +290,8 @@ class VideoWidget(MediaWidget):
                 
                 self.__btn_play.set_images(theme.mb_btn_play_1,
                                            theme.mb_btn_play_2)                
-                self.send_event(self.EVENT_MEDIA_EOF)
                 self.emit_message(msgs.MEDIA_EV_PAUSE)
+                self.send_event(self.EVENT_MEDIA_EOF)
 
 
         elif (cmd == src.OBS_ASPECT):
@@ -273,13 +305,25 @@ class VideoWidget(MediaWidget):
     def __show_error(self, errcode):
     
         if (errcode == self.__player.ERR_INVALID):
-            dialogs.error("Invalid Stream", "Cannot load this stream.")
+            self.call_service(msgs.DIALOG_SVC_ERROR,
+                              "Invalid Stream",
+                              "The media stream is invalid and cannot be loaded.")
+            #dialogs.error("Invalid Stream", "Cannot load this stream.")
         elif (errcode == self.__player.ERR_NOT_FOUND):
-            dialogs.error("Not found", "Cannot find a stream to play.")
+            self.call_service(msgs.DIALOG_SVC_ERROR,
+                              "Media Not Found",
+                              "Cannot find a media stream to play.")
+            #dialogs.error("Not found", "Cannot find a stream to play.")
         elif (errcode == self.__player.ERR_CONNECTION_TIMEOUT):
-            dialogs.error("Timeout", "Connection timed out.")       
+            self.call_service(msgs.DIALOG_SVC_ERROR,
+                              "Connection Timeout",
+                              "A connection timeout occured while loading the media stream.")
+            #dialogs.error("Timeout", "Connection timed out.")       
         elif (errcode == self.__player.ERR_NOT_SUPPORTED):
-            dialogs.error("Not supported", "The media format is not supported.")
+            self.call_service(msgs.DIALOG_SVC_ERROR,
+                              "Unsupported Media Format",
+                              "The format of this media stream is not supported.")
+            #dialogs.error("Not supported", "The media format is not supported.")
 
 
     def __on_set_position(self, pos):
@@ -305,6 +349,8 @@ class VideoWidget(MediaWidget):
         if (self.__current_file):
             bookmarks = self.__progress.get_bookmarks()
             media_bookmarks.set_bookmarks(self.__current_file, bookmarks)
+            self.emit_message(msgs.MEDIA_EV_BOOKMARKED,
+                              self.__current_file, bookmarks)
 
 
     def __scale_video(self):
@@ -336,6 +382,9 @@ class VideoWidget(MediaWidget):
 
         w2 = int(ratio * h)
         h2 = int(w / ratio)
+
+        if (w % 2 == 1): w -= 1
+        if (w2 % 2 == 1): w2 -= 1
          
         #print ratio, w, h, w2, h2, self
         if (w2 > w):
@@ -345,12 +394,15 @@ class VideoWidget(MediaWidget):
             if (w2 > 0): self.__screen.set_size_request(w2, h)
             w2, h2 = w2, h
 
+
         #if (not self.__layout):
         #    self.__layout = self.get_window()
         #    self.__layout.put(self.__screen, x + (w - w2) / 2, y + (h - h2) / 2)
         #else:
         if (self.__layout):
             self.__layout.move(self.__screen, x + (w - w2) / 2, y + (h - h2) / 2)
+            self.__overlay_coords = (x + (w - w2) / 2, y + (h - h2) / 2, w2, h2)
+
         #print  x + (w - w2) / 2, y + (h - h2) / 2, w2, h2
         
         cnt = 0
@@ -358,6 +410,7 @@ class VideoWidget(MediaWidget):
         #    gtk.main_iteration(False)
         #    cnt += 1
         #self.__show_video_screen()
+        self.render()
 
 
     def load(self, item, direction = MediaWidget.DIRECTION_NEXT):
@@ -372,10 +425,7 @@ class VideoWidget(MediaWidget):
             #end if
             
             if (self.__screen.window.xid):
-                #if (uri == self.__uri): return
-                
                 self.__player = mediaplayer.get_player_for_mimetype(item.mimetype)
-                
                 self.__player.set_window(self.__screen.window.xid)
                 try:
                     self.__context_id = self.__player.load_video(uri)
@@ -388,24 +438,30 @@ class VideoWidget(MediaWidget):
                 self.__player.set_volume(mb_config.volume())
                 bookmarks = media_bookmarks.get_bookmarks(item)
                 self.__progress.set_bookmarks(bookmarks)
-                #self.__player.show_text(os.path.basename(uri), 2000)
-                #self.set_title(os.path.basename(uri))                
                 self.__uri = uri
                 self.__current_file = item
                 
-                #self.update_observer(self.OBS_SHOW_PANEL)
-
-
-        self.__show_video_screen()
 
         if (not self.__layout):
+            x, y = self.get_screen_pos()
             self.__layout = self.get_window()
-            self.__layout.put(self.__screen, 0, 0)
+            self.__layout.put(self.__screen, x, y)
+            #self.__screen.set_size_request(10, 10)
+            self.__screen.show()
+            #gtk.gdk.window_process_all_updates()
 
-        if (self.__load_handler):
-            gobject.source_remove(self.__load_handler)
-            
-        self.__load_handler = gobject.idle_add(f)                
+        cnt = 0
+        while (gtk.events_pending() and cnt < 10):
+            gtk.main_iteration(False)
+            cnt += 1
+
+        #if (self.__load_handler):
+        #    gobject.source_remove(self.__load_handler)
+
+        #self.__load_handler = gobject.timeout_add(1500, f)
+        #gobject.idle_add(f)
+        
+        f()
 
 
     def play_pause(self):
@@ -433,10 +489,9 @@ class VideoWidget(MediaWidget):
         mb_config.set_volume(vol)        
         if (self.__player):
             self.__player.set_volume(vol)
-        self.send_event(self.EVENT_MEDIA_VOLUME, vol)
+        self.send_event(self.EVENT_MEDIA_SCALE, vol / 100.0)
 
        
-        
     def decrement(self):
 
         vol = mb_config.volume()
@@ -444,5 +499,26 @@ class VideoWidget(MediaWidget):
         mb_config.set_volume(vol)        
         if (self.__player):
             self.__player.set_volume(vol)
-        self.send_event(self.EVENT_MEDIA_VOLUME, vol)
+        self.send_event(self.EVENT_MEDIA_SCALE, vol / 100.0)
+
+
+    def set_scaling(self, v):
+
+        vol = int(v * 100)
+        mb_config.set_volume(vol)        
+        if (self.__player):
+            self.__player.set_volume(vol)
+        self.send_event(self.EVENT_MEDIA_SCALE, v)
+
+
+    def rewind(self):
+    
+        if (self.__player):
+            self.__player.rewind()
+        
+        
+    def forward(self):
+    
+        if (self.__player):
+            self.__player.forward()
 
