@@ -36,6 +36,9 @@ class ProgressBar(Widget):
         # the current progress as a value between 0.0 and 1.0
         self.__progress = 0
         
+        # the current amount, if any
+        self.__amount = 0.0
+        
         # the current message if any
         self.__current_message = ""
        
@@ -130,6 +133,7 @@ class ProgressBar(Widget):
                 del self.__bookmarks[self.__dragged_bookmark]
                 self.render()
                 print "DELETED BOOKMARK"
+                self.__dragged_bookmark = -1
                 self.send_event(self.EVENT_BOOKMARK_CHANGED)
             
             # has the bookmark been selected?
@@ -193,6 +197,7 @@ class ProgressBar(Widget):
         pmap = TEMPORARY_PIXMAP
         pmap.copy_pixmap(self.__bg_pmap, 0, 0, 0, 0, w, h)
         self.__render_progress(pmap)
+        self.__render_amount(pmap)
         self.__render_bookmarks(pmap)
         self.__render_message(pmap)
         screen.copy_pixmap(pmap, 0, 0, x, y, w, h)
@@ -209,15 +214,27 @@ class ProgressBar(Widget):
         #end if
 
 
+    def __render_amount(self, pmap):
+    
+        w, h = self.get_size()
+        
+        amount_width = w * self.__amount
+        if (1 < amount_width < w):
+            height = 32
+            y = (h - height) / 2
+            pmap.fill_area(0, y, amount_width, height, "#ff000040")
+
+
     def __render_bookmarks(self, pmap):
     
         w, h = self.get_size()
         for bm in self.__bookmarks:
-            bm_pos = w * bm
-            #pmap.fill_area(bm_pos - 1, 0, 2, 32, "#ff0000")
-            pmap.draw_pixbuf(theme.mb_progress_bookmark,
-                        bm_pos - theme.mb_progress_bookmark.get_width() / 2,
-                        (h - theme.mb_progress_bookmark.get_height()) / 2)
+            if (bm > 0):
+                bm_pos = w * bm
+                #pmap.fill_area(bm_pos - 1, 0, 2, 32, "#ff0000")
+                pmap.draw_pixbuf(theme.mb_progress_bookmark,
+                            bm_pos - theme.mb_progress_bookmark.get_width() / 2,
+                            (h - theme.mb_progress_bookmark.get_height()) / 2)
         #end for
         
         
@@ -251,6 +268,7 @@ class ProgressBar(Widget):
         self.__bookmarks = bookmarks[:]
         self.render()
 
+
     def get_bookmarks(self):
         """
         Returns the list of bookmarks.
@@ -275,7 +293,26 @@ class ProgressBar(Widget):
         if (not self.may_render()): return
         if (total == 0): return
 
-        self.__progress = pos / float(total)
-        self.render()
+        new_progress = pos/float(total)
+        if (abs(self.__progress - new_progress) > 0.001):            
+            self.__progress = new_progress
+            self.render()
         #self.send_event(self.EVENT_CHANGED, self.__progress * 100)
 
+
+    def set_amount(self, amount):
+        """
+        Sets the amount as a value between 0.0 and 1.0.
+        @since: 0.96.5
+        
+        @param amount: the amount value
+        """
+
+        self.set_message("")
+        if (self.__is_dragging and self.__dragged_bookmark == -1): return
+        if (not self.may_render()): return
+        
+        if (abs(amount - self.__amount) > 0.001):
+            self.__amount = amount
+            self.render()
+
